<?php
/**  
 * OpenVCP Webinterface
 * Copyright (C) 2006 Gerrit Wyen <gerrit@t4a.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 * @author  Oliver Werner / OpenVCP Team
 * @version 0.3.0.1
 * @package OpenVCP-v0.3
 */

class Node {

	private $nodeid = 0;
	private $nodeinfo = null;
	private $adminList = null;
	private $DB = null;
	private $XML = null;
	private $SOCKET = null;

    function __construct($node) {
    	$this->DB =& OpenVCP::call('DB', 'getReference');
    	if(is_numeric($node) && $node > 0)
    	{
    		$node = $this->DB->select($this->DB->table_nodes, NULL, NULL, array('id' => $node));
    	}
    	else if(is_string($node) && $node != "")
    	{
    		$node = $this->DB->select($this->DB->table_nodes, NULL, NULL, array('name' => $node));
    	}
    	else
    	{
    		OpenVCP::coreError("node does not exist");
    	}
    	$this->SOCKET =& OpenVCP::call('Socket', 'getReference');
		$this->XML	=& OpenVCP::call('XML', 'getReference');
   		$this->nodeinfo = $node[0];
    }
    
    public function getId()
    {
    	return $this->nodeinfo['id'];
    }
    
    public function getName()
    {
    	return $this->nodeinfo['name'];
    }
    
    public function getIP()
    {
    	return $this->nodeinfo['ip'];
    }

    public function getPort()
    {
    	return $this->nodeinfo['port'];
    }

    public function getUsername()
    {
    	return $this->nodeinfo['username'];
    }
    
    public function getPassword()
    {
    	return $this->nodeinfo['password'];
    }

	public function getTLS()
	{
		return ($this->nodeinfo['tls'] == 'y' ? true : false);
	}
	
	/**
	 * returns a list of admins which have access to
	 * the specific node
	 */
	public function getAdmins()
	{
		if($this->adminList == null)
			$this->adminList = $this->DB->select($this->DB->table_admin_nodes, NULL, array('adminid'), array('nodeid' => $this->nodeinfo['id']));
		return $this->adminList;
	}

	/**
	 * checks whether a admin has access to the node or not
	 * 
	 * @param string $adminName
	 */
	public function checkAdmin($adminName)
	{
		if(($adminList = $this->getAdmins()) != null)
		{
			foreach($this->getAdmins() as $key => $value)
			{
				if(in_array($adminName, $value))
					return true;
			}
			return false;
		}
		else
			return false;
	}
	
	/**
	 * Returns an array containing the guest names
	 * 
	 * @return array $guestList
	 */
	public function getGuestList()
	{
		$request = $this->XML->newRequestObject("node.guest_list");
		$response = $this->SOCKET->socket_exec($this->nodeinfo['id'], $request);
		$list = $this->XML->getNextArray($response);
		sort($list);
		return $list;
	}
	
	/**
	 * returns the current Traffic of the node
	 * 
	 * @param int $month
	 * @param int $year
	 * @return array traffic of the month
	 */
	public function getTraffic($month, $year)
	{
		$request = $this->XML->newRequestObject("node.traffic_total");
		$this->XML->addValue($request, "int", $month);
		$this->XML->addValue($request, "int", $year);
		$response = $this->SOCKET->socket_exec($this->nodeinfo['id'], $request);
		return $this->XML->getNextStruct($response);
	}
	
	/**
	 * return the current Load of the node
	 * 
	 * @param boolean $format
	 * @return string load
	 */
	public function getLoad($format = false)
	{
		$request = $this->XML->newRequestObject("node.get_load");
		$response = $this->SOCKET->socket_exec($this->nodeinfo['id'], $request);
		$load = $this->XML->getValues($response, "string");
		if($format)
		{
			$data = explode(" ", $load[0]);
			$returndata = "";
			for($a = 0; $a < 3; $a++)
			{
				if($data[$a] < 1) $returndata .= "<font color=\"green\">".$data[$a]."</font>"; 
				else if($data[$a] < 5) $returndata .= "<font color=\"#FFCC00\">".$data[$a]."</font>";
				else $returndata .= "<font color=\"red\">".$data[$a]."</font>";
				$a < 2 ? $returndata .= "|" : $returndata .= "";
			}
			return $returndata;
		}
		else
			return $load[0];
	}
	
	/**
	 * returns the current uptime of the node
	 * 
	 * @param boolean $format
	 * @return string uptime
	 */
	public function getUptime($format)
	{
		$request = $this->XML->newRequestObject("node.get_uptime");
		$response = $this->SOCKET->socket_exec($this->nodeinfo['id'], $request);
		$uptime = $this->XML->getValues($response, "string");
		if($format)
		{
		 	$input = explode(" ", $uptime[0]);
			if($input[0] != 0)
			{
			 	$reference = 3600 * 24;
			 	$days = 0;
			 	while($input[0] > ($reference * $days))
			 	{
			 		$days++;
			 	}
			 	$days--;
			 	$input[0] = $input[0] - ($days * $reference);
			 	$hours = 0;
			 	while($input[0] > (3600 * $hours))
			 	{
			 		$hours++;
			 	}
			 	$hours--;
			 	$input[0] = $input[0] - ($hours * 3600);
			 	$mins = 0;
			 	while($input[0] > ($mins * 60))
			 	{
			 		$mins++;
			 	}
			 	$mins = ($mins < 10) ? "0".$mins : $mins;
			 	$hours = ($hours < 10) ? "0".$hours : $hours;
			}
			else
			{
				$days = 0;
				$hours = "00";
				$mins = "00";
			}
		 	return $days." "._("days").", ".$hours." ". _("hours") ." ".$mins." "._("minutes");
		}
		else
			return $uptime[0];
	}
	
	/**
	 * returns the version of the node
	 * 
	 * @return string version
	 */
	public function getVersion()
	{
		$request = $this->XML->newRequestObject("node.version");
		$response = $this->SOCKET->socket_exec($this->nodeinfo['id'], $request);
		$version = $this->XML->getValues($response, "string");
		return 	$version[0];
	}
	
	/**
	 * returns the current memory usage
	 * 
	 * @return array memory usage
	 */
	public function getMemory()
	{
		$request = $this->XML->newRequestObject("node.get_mem");
		$response = $this->SOCKET->socket_exec($this->nodeinfo['id'], $request);
		return $this->XML->getNextStruct($response);	
	}
	
	/**
	 * returns the current space used
	 * 
	 * @return array space usage
	 */
	public function getSpace()
	{
		$request = $this->XML->newRequestObject("node.get_space");
		$response = $this->SOCKET->socket_exec($this->nodeinfo['id'], $request);
		return $this->XML->getNextStruct($response);
	}
	
	/**
	 * adds a admin with unlimited access to the node
	 * 
	 * @param int $adminId
	 */
	public function addAdmin($adminId)
	{
		$this->DB->insert($this->DB->table_admin_nodes, array('',  $adminId, $this->nodeinfo['id']), array('id', 'adminid', 'nodeid'));
	}
	
	/**
	 * deletes a admin from the node
	 * 
	 * @param int $adminId
	 */
	public function deleteAdmin($adminId)
	{
		$this->DB->delete($this->DB->table_admin_nodes, array('adminid' => $adminId, 'AND', 'nodeid' => $this->nodeinfo['id']));
	}
	
	/**
	 * get vserver count of this node
	 * 
	 * @return count of vserver
	 */
	
	public function getVServerCount()
	{
		$DB =& OpenVCP::call('DB', 'getReference');
		$vservers = $DB->select($DB->table_vservers, NULL, NULL, array('nodeid' => $this->nodeinfo['id']));
		return count($vservers);
	}

	/**
	 * returns a array containing all nodes
	 * 
	 * @return array id and name of all nodes
	 */
	public static function getNodeList()
	{
		$DB =& OpenVCP::call('DB', 'getReference');
		return $DB->select($DB->table_nodes, NULL, array('id', 'name'));
	}
	
}
?>
